#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
#
# Template script for configuring a service to use the servicetool for gLite
# Joachim Flammer <Joachim.Flammer@cern.ch>
# Version info: $Id: gliteRgmaServicetool.py,v 1.16.2.11 2006/03/17 18:52:08 flammer Exp $
#
#####################################################################################
import os,string,socket, pwd
import sys
import getopt
import commands
from gLiteInstallerLib import gLib as GLib
from xml.dom.minidom import getDOMImplementation    
from xml.dom.ext import PrettyPrint

global glib

class gliteRgmaServicetool:

        
    #################################################################
    # init
    #
    # initialization
    #################################################################
    def __init__(self):        

        self.verbose = 0
        self.glib = GLib()
        
        self.servicesPath = '%s/etc/rgma-servicetool/services' % os.environ['GLITE_LOCATION']        
        

    #################################################################
    # configureServicetool
    #
    # configures servicetool
    #
    # return 0 ok
    #        1 error
    #################################################################
    def configure(self, glib):
                                
        # ------------------------------------------------------------
        # configure servicetool itself
        # ------------------------------------------------------------
        if os.system("%s/etc/config/scripts/glite-rgma-servicetool-config.py --subservice --configure" % os.environ['GLITE_LOCATION']):
            return 1
        
        # ------------------------------------------------------------
        # configure service instances
        # ------------------------------------------------------------
        if self.configureServices(glib):
            return 1
        
        return 0    
    
    
    #################################################################
    # configureService
    #
    # configure one service to use service tool
    #
    # params glib        Parameter dictionary
    #
    # return 0 ok
    #        1 error
    #################################################################
    def configureServices(self, glib):        

        self.glib = glib
        
        if self.configureServiceInstances():
            return 1

        return 0            


    #################################################################
    # start
    #
    # function to start service publisher
    # 
    # return
    #################################################################
    def start(self):
        
        errorflag = os.system("%s/etc/config/scripts/glite-rgma-servicetool-config.py --start --subservice" % os.environ['GLITE_LOCATION'])
    
        return errorflag

        
    #################################################################
    # stop
    #
    # function to stop service publisher
    # 
    # return
    #################################################################
    def stop(self):
        
        errorflag = os.system("%s/etc/config/scripts/glite-rgma-servicetool-config.py --stop --subservice" % os.environ['GLITE_LOCATION'])
    
        return errorflag

        
    #################################################################
    # status
    #
    # function to stop service publisher
    # 
    # return
    #################################################################
    def status(self):
        
        errorflag = os.system("%s/etc/config/scripts/glite-rgma-servicetool-config.py --status" % os.environ['GLITE_LOCATION'])
    
        return errorflag


    #################################################################
    # configureServiceInstances
    # 
    # function to configure the services to be published by RGMA servicetool
    # function reads specified configuration file and gets all instances of 
    # 'service'-type 'rgma-servicetool' and creates/removes the corresponding 
    # service files
    # 
    # params glib glib library
    #
    # return 0 ok
    #        1 error
    #################################################################
    def configureServiceInstances(self):
     
        print "Configuring services to use R-GMA servicetool ..."
         
        # get all instances for R-GMA servicetool
        instances = self.glib.getInstances('rgma-servicetool')

        # configure instances
        if len(instances) > 0:
            for instanceName in instances:
                
                # configure one instance
                if (self.configureServiceInstance(instanceName) !=0):
                    # something went wrong
                    print "Configuring services to use R-GMA servicetool             ",
                    self.glib.printFailedMessage()
                    return 1 
                    
        # all ok
        print "Configuring services to use R-GMA servicetool             ",
        self.glib.printOkMessage()
        return 0  
    
    
    #################################################################
    # configureServiceInstance
    # 
    # function to configure/remove one service for the servicetool
    # using instances
    #
    # params serviceId id of service
    #
    # return 0 ok
    #        1 error
    #################################################################
    def configureServiceInstance(self, serviceId):
    
        instanceParams = {}
        
        # remove 'rgma-servicetool:' prefix
        self.serviceId = serviceId[len('rgma-servicetool:'):]
        
        # get instance parameters
        try:
            instanceParams = self.glib.getInstanceParams(serviceId)
        except ValueError, msg:
            self.glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
            sys.exit(1)
        except LookupError, msg:
            print msg        
        
        # check existance 
        if not(instanceParams.has_key('rgma.servicetool.enable')):
            self.parameterErrorMessage("rgma.servicetool.enable", self.serviceId, "Configuring instance '%s' of service for servicetool" % self.serviceId, "ERROR")
            return 1
        else:
            enabled = instanceParams['rgma.servicetool.enable']
        
        # check enable status
        if (enabled == 'true'):

            # ------------------------------------------------------------
            # get non-optional parameters
            # ------------------------------------------------------------
                        
            # endpoint
            if instanceParams.has_key('rgma.servicetool.url_endpoint'):
                self.serviceEndpoint = instanceParams['rgma.servicetool.url_endpoint']
            else:
                self.parameterErrorMessage("rgma.servicetool.url_endpoint", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
            
            # type
            if instanceParams.has_key('rgma.servicetool.service_type'):
                self.serviceType = instanceParams['rgma.servicetool.service_type']
            else:
                self.parameterErrorMessage("rgma.servicetool.service_type", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
            
            # version
            if instanceParams.has_key('rgma.servicetool.service_version'):
                self.serviceVersion = instanceParams['rgma.servicetool.service_version']
            else:
                self.parameterErrorMessage("rgma.servicetool.service_version", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
                
            # publish interval
            if instanceParams.has_key('rgma.servicetool.publish_interval'):
                self.servicePublishInterval = instanceParams['rgma.servicetool.publish_interval']
            else:
                self.parameterErrorMessage("rgma.servicetool.publish_interval", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
            
            # status script
            if instanceParams.has_key('rgma.servicetool.status_script'):
                self.serviceStatusScript = instanceParams['rgma.servicetool.status_script']
            else:
                self.parameterErrorMessage("rgma.servicetool.status_script", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
                
            # status interval
            if instanceParams.has_key('rgma.servicetool.status_interval'):
                self.serviceStatusInterval = instanceParams['rgma.servicetool.status_interval']
            else:
                self.parameterErrorMessage("rgma.servicetool.status_interval", self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
                
            # wsdl url
            if instanceParams.has_key('rgma.servicetool.url_wsdl'):
                self.serviceWsdlUrl = instanceParams['rgma.servicetool.url_wsdl']
            else:
                self.parameterErrorMessage("rgma.servicetool.url_wsdl",  self.serviceId, "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")
                return 1
                
            # semantics url
            if instanceParams.has_key('rgma.servicetool.url_semantics'):
                self.serviceSemanticsUrl = instanceParams['rgma.servicetool.url_semantics']
            else:
                self.parameterErrorMessage("rgma.servicetool.url_semantics",  "Configuring service '%s' for servicetool" % self.serviceId, "FATAL")        
                return 1
                            
            # ------------------------------------------------------------
            # get optional parameters
            # ------------------------------------------------------------
            
            # name
            if instanceParams.has_key('rgma.servicetool.name'):
                self.serviceName = instanceParams['rgma.servicetool.name']
            else:
                self.serviceName = ""            
            
            # vo list => optional
            self.serviceVos = []
            if instanceParams.has_key('rgma.servicetool.vo'):
                if len(instanceParams['rgma.servicetool.vo']) > 0:
                    for item in instanceParams['rgma.servicetool.vo']:
                        if item != "":
                            self.serviceVos.append(item)
            
            # associatedServices list => optional
            self.serviceAssociated = []
            if instanceParams.has_key('rgma.servicetool.associatedService'):
                if len(instanceParams['rgma.servicetool.associatedService']) > 0:
                    for item in instanceParams['rgma.servicetool.associatedService']:
                        if item != "":
                            self.serviceAssociated.append(item)
            
            # extra parameter list => optional
            self.serviceParams = []
            if instanceParams.has_key('rgma.servicetool.param'):
                if len(instanceParams['rgma.servicetool.param']) > 0:
                    for item in instanceParams['rgma.servicetool.param']:
                        if item.find('=') != -1:
                            self.serviceParams.append(item)
                        else:
                            if item !="":
                                self.glib.printErrorMessage("Parameter 'rgma.servicetool.param' has incorrect format.")
                                self.glib.printErrorMessage("Format is key=value. Parameter given is: '%s'." % item)
                                return 1
                                                    
            # check if service id exists already:
            if self.serviceFileExists(serviceId):
                self.removeServiceFile()
                
            # enable service for servicetool: create service file            
            return self.createServiceFile()
                          
        elif (enabled == 'false'):
        
            # disable service for servicetool: remove service file
            return self.removeServiceFile(self.serviceId)

        else:
    
            # wrong enable parameter set
            self.glib.printErrorMessage("Configuring service for servicetool failed:")
            print "        Parameter 'rgma.servicetool.enable' for instance '%s' of service 'rgma-servicetool' uncorrectly set." % name
            print "        Must be (true|false) - value is '%s' " % instanceParams['rgma.servicetool.enable']
            print "        Please correct configuration file and redo configuration."
            return 1                        


    #################################################################
    # serviceFileExists
    #
    # function to check if service file for service id exists alreayd
    #
    # params serviceId  id of service
    #
    # return 0 not found
    #        1 found
    #################################################################
    def serviceFileExists(self, serviceId):

        found = 0
        
        # loop over all files in directory
        serviceList = os.listdir(self.servicesPath)

        for entry in serviceList:
            file = open('%s/%s' % (self.servicesPath,entry), 'r')
            for line in file.readlines():
                if line.startswith("Name="):
                    entryServiceId = line[5:-1]
                    if entryServiceId == serviceId:
                        file.close()
                        found = 1
                        break
            file.close()
                    
        if found == 1:                    
            return 1
        else:
            return 0


    #################################################################
    # createServiceFile
    #
    # function to create a service file for the servicetool
    #
    # return 0 ok
    #        1 error
    #################################################################
    def createServiceFile(self):
                
        print "Creating configuration file for service id '%s' \n   to be published via R-GMA servicetool                  " % self.serviceId,
    
        serviceFile = '%s/%s.service' % (self.servicesPath, self.serviceId)
    
        try:
            os.system("/bin/touch %s" % serviceFile)
            file = open(serviceFile, 'w')
            file.write('Name=%s\n'            % self.serviceId)
            if self.serviceName != "":
                file.write('ReadableName=%s\n'    % self.serviceName)
            file.write('Endpoint=%s\n'        % self.serviceEndpoint)
            file.write('Type=%s\n'            % self.serviceType)
            file.write('Version=%s\n'         % self.serviceVersion)
            file.write('PublishInterval=%s\n' % self.servicePublishInterval)
            file.write('StatusScript=%s\n'    % self.serviceStatusScript)
            file.write('StatusInterval=%s\n'  % self.serviceStatusInterval)
            file.write('WSDL=%s\n'            % self.serviceWsdlUrl)
            file.write('Semantics=%s\n'       % self.serviceSemanticsUrl)
            
            # vo list (optional)
            if len(self.serviceVos) != 0:
                for vo in self.serviceVos:
                    file.write('VO=%s\n' % vo)                        
    
            # associated services (optional)
            if len(self.serviceAssociated) != 0:
                for service in self.serviceAssociated:
                    file.write('AssociatedService=%s\n' % service)
                    
            # service params (optional)
            if len(self.serviceParams) != 0:
                for parameter in self.serviceParams:
                    param = parameter.split("=")
                    if len(param) != 2:
                        self.glib.printFailedMessage()
                        self.glib.printErrorMessage("Parameter '%s' does not have key=value format")
                        return 1
                    paramName  = param[0]
                    paramValue = param[1]
                    file.write('%s=%s\n' % (paramName, paramValue))
                    
            file.close()
    
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                self.glib.printFailedMessage()
                self.glib.printErrorMessage("File '%s' is not writable." % serviceFile)
                return 1
            else:
                print ex.strerror + "        ",
                self.glib.printFailedMessage()
                return 1
        
        self.glib.printOkMessage()
        return 0   


    #################################################################
    # removeServiceFile
    # 
    # function to remove a service file for the servicetool
    # 
    # params serviceId id of service to be removed
    #
    # return 0 ok
    #        1 error
    #################################################################
    def removeServiceFile(self, serviceId):
            
        print "Removing configuration file for service id '%s' \n from R-GMA servicetool                                  " % serviceId,

        found = 0
        
        # loop over all files in directory
        serviceList = os.listdir(self.servicesPath)

        for entry in serviceList:
            file = open('%s/%s' % (self.servicesPath,entry), 'r')
            for line in file.readlines():
                if line.startswith("Name="):
                    entryServiceId = line[5:-1]
                    if entryServiceId == serviceId:
                        file.close()
                        os.system('rm %s/%s' % (self.servicesPath,entry))
                        found = 1
                        break
            file.close()             
                    
        if found == 1:                    
            self.glib.printOkMessage()
            return 0
        else:
            # file does not exist: nothing to be done
            self.glib.printFailedMessage()
            self.glib.printWarningMessage("Service id does not exist")
            return 1
            
        
    #################################################################
    # showServices
    #
    # show list of services that are published with their parameters
    #
    # return 0 ok
    #        1 error
    #################################################################
    def showServices(self):

        # print list of services that are published
        print "R-GMA servicetool is configured to publish the following services:\n"
        
        items = os.listdir(self.servicesPath)
        for item in items:
            file = open("%s/%s" % (self.servicesPath, item), 'r')
            if item.endswith(".service"):
                item = item[:-8]            
            id              = ""
            version         = ""
            name            = ""
            type            = ""
            endpoint        = ""
            wsdl            = ""
            semantics       = ""
            statusScript    = ""
            publishInterval = ""
            statusInterval  = ""
            vo              = []
            associatedServices = []
            param = []
            
            # get entries
            for line in file.readlines():
                if line.startswith("Name"):
                    value = line.split("=")
                    if len(value) > 1:
                        id = value[1][:-1]
                elif line.startswith("ReadableName"):
                    value = line.split("=")
                    if len(value) > 1:
                        name = value[1][:-1]
                elif line.startswith("Version"):
                    value = line.split("=")
                    if len(value) > 1:
                        version = value[1][:-1]
                elif line.startswith("Type"):
                    value = line.split("=")
                    if len(value) > 1:
                        type = value[1][:-1]
                elif line.startswith("StatusScript"):
                    startPos = line.find("=")
                    if startPos != -1:
                        statusScript = line[startPos+1:-1]
                elif line.startswith("Endpoint"):
                    value = line.split("=")
                    if len(value) > 1:
                        endpoint = value[1][:-1]
                elif line.startswith("WSDL"):
                    value = line.split("=")
                    if len(value) > 1:
                        wsdl = value[1][:-1]
                elif line.startswith("Semantics"):
                    value = line.split("=")
                    if len(value) > 1:
                        semantics = value[1][:-1]
                elif line.startswith("PublishInterval"):
                    value = line.split("=")
                    if len(value) > 1:
                        publishInterval = value[1][:-1]
                elif line.startswith("StatusInterval"):
                    value = line.split("=")
                    if len(value) > 1:
                        statusInterval = value[1][:-1]
                elif line.startswith("VO"):
                    value = line.split("=")
                    if len(value) > 1:
                        vo.append(value[1][:-1])
                elif line.startswith("AssociatedService"):
                    value = line.split("=")
                    if len(value) > 1:
                        associatedServices.append(value[1][:-1])
                else:
                    param.append(line[:-1])
            
            print "  \033[34m[ID: \x1b[0m%s\033[34m]\x1b[0m" % id
            if name != "":
                print "     \033[34mName:\x1b[0m              %s " % name
            print "     \033[34mType:\x1b[0m              %s " % type
            print "     \033[34mVersion:\x1b[0m           %s " % version
            print "     \033[34mStatusScript:\x1b[0m      %s " % statusScript
            print "     \033[34mPublishInterval:\x1b[0m   %s s" % publishInterval
            print "     \033[34mStatusInterval:\x1b[0m    %s s" % statusInterval
            print "     \033[34mEndpoint:\x1b[0m          %s " % endpoint
            print "     \033[34mWsdlURL:\x1b[0m           %s " % wsdl
            print "     \033[34mSemanticsURL:\x1b[0m      %s " % semantics
            i = 0
            for s in associatedServices:
                if (i==0):
                    print "     \033[34mAssociatedService:\x1b[0m %s " % s
                else:
                    print "                        %s " % s
                i = i+1

            i = 0
            for s in vo:
                if (i==0):
                    print "     \033[34mVo:\x1b[0m                %s " % s
                else:
                    print "                        %s " % s                    
                i = i+1   

            i = 0
            for s in param:
                if (i==0):
                    print "     \033[34mParam:\x1b[0m             %s " % s
                else:
                    print "                        %s " % s                    
                i = i+1   
            print
            file.close()

    #################################################################
    # parameterErrorMessage
    #
    # function to print out error message
    # 
    # params parameterName name of parameter
    #        operation     text of operation
    #        severity      severity ('ERROR', 'FATAL')
    #
    # return 0 ok
    #        1 error
    #################################################################
    def parameterErrorMessage(self, parameterName, serviceId, operation, severity):
    
        print "[\033[31m%s\x1b[0m] %s failed: " % (severity, operation)
        print "[\033[31mERROR\x1b[0m] Parameter '%s' of service instance '%s' for 'rgma-servicetool'" % (parameterName, serviceId)
        print "[\033[31mERROR\x1b[0m] uncorrectly set or missing. Please correct configuration file and redo configuration."
        
        if (severity=="FATAL"):
            sys.exit(1)
        else:
            return 0


    #################################################################
    # createRgmaUser
    #
    # create RGMA user
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaUser(self, glib):
    
        # ------------------------------------------------------------
        # create rgma group and user
        # ------------------------------------------------------------

        if self.verbose == 0:
            print "Creating R-GMA user and group                             ",
        else:
            print "Creating R-GMA user and group ...                         "
        
        # get advanced and system parameters
        self.rgma_user_name         = glib.params['rgma.user.name']
        self.rgma_user_group        = glib.params['rgma.user.group']
        self.user_certificates_path = glib.params['user.certificate.path']
        self.host_key_file          = glib.params['host.key.file']
        self.host_certificate_file  = glib.params['host.certificate.file']

        if glib.add_user(self.rgma_user_name,self.rgma_user_group) == -1:
            glib.printFailedMessage()
            return 1

        if self.verbose == 0:            
            glib.printOkMessage()
        else:
            print "Creating R-GMA user and group                             ",
            glib.printOkMessage()

        # ------------------------------------------------------------
        # Updating host key and certificate for R-GMA
        # ------------------------------------------------------------

        self.rgma_user_home_dir = pwd.getpwnam(self.rgma_user_name)[5]

        self.user_cert_filename = "hostcert.pem"
        self.user_key_filename  = "hostkey.pem"

        # create dir user_certificates_path if it does not exist
        if not os.path.exists("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path)):
            os.mkdir("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path),0755)

        # copy key and cert file        
        errorcode = 0
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_key_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_certificate_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
        if errorcode:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not copy certificate and key for user '%s'" % self.rgma_user_name)
            glib.printErrorMessage("to files '%s/%s/%s' and '%s/%s/%s'" % \
                (self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename, \
                  self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))
            return 1
        
        # change owner of key and cert file        
        errorcode = 0
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
        
        if errorcode:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not change access rights of certificate and key for user '%s:%s'" % \
                (self.rgma_user_name, self.rgma_user_group))
            glib.printErrorMessage("for file '%s/%s/hostkey.pem' and '%s/%s/hostcert.pem'" % \
                (self.rgma_user_home_dir, self.user_certificates_path))
            return 1

        if self.verbose:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printOkMessage()

        return 0


#################################################################
# Wrapper class to create the rgma servicetool instances.
#
# Remarks: The following parameters MUST be set via the corresponding
#          setter functions:
#             serviceType
#             serviceName
#             statusScript
#             serviceVersion
#          The rest of the parameters are optional.
#
# Usage: 
#         from gliteRgmaServicetool import gliteRgmaServicetoolInstance
#         from gliteRgmaServicetool import gliteRgmaServicetool
#    
#         # --------------------------------------
#         # Instantiate the rgma-servicetool class
#         # --------------------------------------
#         rgmaServicetool = gliteRgmaServicetool()
#         rgmaServicetool.verbose = verbose
#
#         # --------------------------------------
#         # Create one instance
#         # --------------------------------------
#         servictoolInstance = gliteRgmaServicetoolInstance(glib, instanceName)
#         # set params
#         servicetoolInstance.setServiceName(name)
#         servicetoolInstance.setServiceType(type)
#         servicetoolInstance.setServiceVersion(version)
#         servicetoolInstance.setStatusScript(scriptLocation)
#          ...
#          # add extra params like endpoint if you want
#          ...
#         # add to configuration
#         servicetoolInstance.add()
#
#         # --------------------------------------
#         # Configure rgma-servicetool
#         # --------------------------------------
#         if rgmaServicetool.configure(glib):
#              return 1
#
#################################################################
class gliteRgmaServicetoolInstance:
    
    #################################################################
    # init
    #
    # initialization
    #
    # params glib         glib library (to add parameters)
    #        serviceId    unique id of service in form:
    #                     HOSTNAME_VONAME_SERVICETYPE_EXTRAVALUES
    #################################################################
    def __init__(self, glib, serviceId):        

        self.glib = glib
        self.verbose = 0
        self.instanceType = "rgma-servicetool"
    
        # non default parameters
        self.serviceId          = serviceId
        self.serviceType        = ""
        self.serviceVersion     = ""
        self.statusScript       = ""        
        
        # default parameters
        self.serviceEndpoint    = ""
        self.serviceName        = ""
        self.publishInterval    = ""
        self.statusInterval     = ""
        self.wsdlUrl            = ""
        self.semanticsUrl       = ""
        self.enableFlag         = ""
        self.associatedServices = []
        self.serviceParams      = []
        self.vos                = []            
    
    #################################################################
    # setServiceName
    #
    # set the service name
    #
    # params serviceName human readable name of service (must not be unique)
    #
    #################################################################    
    def setServiceName(self, serviceName):        
        self.serviceName = serviceName
        
    #################################################################
    # setServiceType
    #
    # set the service name
    #
    # params serviceType service type - unique for service of one type
    #
    #################################################################    
    def setServiceType(self, serviceType):        
        self.serviceType = serviceType
        
    #################################################################
    # setServiceVersion
    #
    # set the service version
    #
    # params serviceVersion version of service in form major.minor.bug 
    #
    #################################################################    
    def setServiceVersion(self, serviceVersion):
        self.serviceVersion = "%s" % serviceVersion
    
    #################################################################
    # setStatusScript
    #
    # set the status script
    #
    # params statusScript status script 
    #
    #################################################################    
    def setStatusScript(self, statusScript):
        self.statusScript = statusScript
    
    #################################################################
    # setServiceEndpoint
    #
    # set the service endpoint
    #
    # params endpoint  service endpoint 
    #
    #################################################################    
    def setEndpoint(self, endpoint):
        self.serviceEndpoint = endpoint
    
    #################################################################
    # setStatusInterval
    #
    # set the status publish interval
    #
    # params statusInterval    time interval for status publishing (in seconds)
    #
    #################################################################    
    def setStatusInterval(self, statusInterval):        
        self.statusInterval = "%s" % statusInterval       

    #################################################################
    # setPublishInterval
    #
    # set the publish interval for service infos
    #
    # params publishInterval    time interval for publishing service infos (in seconds)
    #
    #################################################################    
    def setPublishInterval(self, publishInterval):        
        self.publishInterval = "%s" % publishInterval       
        
    #################################################################
    # setWsdlUrl
    #
    # set the wsdl url
    #
    # params wsdlUrl    url of wsdl
    #
    #################################################################    
    def setWsdlUrl(self, wsdlUrl):        
        self.wsdlUrl = wsdlUrl    
    
    #################################################################
    # setSemanticsUrl
    #
    # set the url of the semantics
    #
    # params semanticsUrl    url of semantics
    #
    #################################################################    
    def setSemanticsUrl(self, semanticsUrl):        
        self.semanticsUrl = semanticsUrl
        
    #################################################################
    # setAssociatedServices
    #
    # set the associated services
    #
    # params associatedServices    list of associatedServices
    #
    #################################################################    
    def setAssociatedServices(self, associatedServices):        
        self.associatedServices = associatedServices

    #################################################################
    # setParams
    #
    # set the extra params
    #
    # params serviceParams    list of parameters in key=value form
    #
    #################################################################    
    def setParams(self, serviceParams):        
        self.serviceParams = serviceParams

    #################################################################
    # setVos
    #
    # set the vos
    #
    # params vos    list of vos
    #
    #################################################################    
    def setVos(self, vos):        
        self.vos = vos

    #################################################################
    # setEnableFlag
    #
    # set the activate entry
    #
    # params enableFlag    enableFlag (true|false)
    #
    #################################################################    
    def setEnableFlag(self, enableFlag):
        self.enableFlag = enableFlag

    #################################################################
    # add
    #
    # add instance to configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################    
    def add(self):
                            
        if self.serviceId == "":
            self.glib.printErrorMessage("Service Id not set for R-GMA servicetool instance.")
            return 1
        
        # create sub dom tree
        impl = getDOMImplementation()
        doc = impl.createDocument(None, "instance", None)
        doc.documentElement.setAttribute("name",    self.serviceId)
        doc.documentElement.setAttribute("service", self.instanceType)
        
        # create parameters element
        parametersElement = doc.createElement("parameters")
        doc.documentElement.appendChild(parametersElement)

        # check if non-default params have been set
        if self.serviceType != "":
            # create serviceType element
            serviceTypeElement = doc.createElement("rgma.servicetool.service_type")
            serviceTypeElement.setAttribute("value", self.serviceType)
            parametersElement.appendChild(serviceTypeElement)
        else:
            self.glib.printErrorMessage("Service Type not set for R-GMA servicetool instance '%s'" % self.serviceId)
            return 1

        if self.serviceVersion != "":
            # create serviceVersion element
            serviceVersionElement = doc.createElement("rgma.servicetool.service_version")
            serviceVersionElement.setAttribute("value", self.serviceVersion)
            parametersElement.appendChild(serviceVersionElement)
        else:
            self.glib.printErrorMessage("Service Version not set for R-GMA servicetool instance '%s'" % self.serviceId)
            return 1

        if self.statusScript != "":
            # create statusScript element
            statusScriptElement = doc.createElement("rgma.servicetool.status_script")        
            statusScriptElement.setAttribute("value", self.statusScript)
            parametersElement.appendChild(statusScriptElement)
        else:
            self.glib.printErrorMessage("Service status script not set for R-GMA servicetool instance '%s'" % self.serviceId)
            return 1

        if self.serviceEndpoint != "":
            # create serviceEndpoint element
            serviceEndpointElement = doc.createElement("rgma.servicetool.url_endpoint")
            serviceEndpointElement.setAttribute("value", self.serviceEndpoint)
            parametersElement.appendChild(serviceEndpointElement)
        else:
            self.glib.printErrorMessage("Service endpoint not set for R-GMA servicetool instance '%s'" % self.serviceId)
            return 1

        # activate flag
        if self.enableFlag != "":
            # create activate element
            enableFlagElement = doc.createElement("rgma.servicetool.enable")
            enableFlagElement.setAttribute("value", self.enableFlag)
            parametersElement.appendChild(enableFlagElement)
        
        # create associatedService element
        if self.serviceName != "":
            # create serviceName element
            serviceNameElement = doc.createElement("rgma.servicetool.name")
            serviceNameElement.setAttribute("value", self.serviceName)
            parametersElement.appendChild(serviceNameElement)
        
        if self.publishInterval != "":
            # create publishInterval element
            publishIntervalElement = doc.createElement("rgma.servicetool.publish_interval")
            publishIntervalElement.setAttribute("value", self.publishInterval)
            parametersElement.appendChild(publishIntervalElement)
        
        if self.statusInterval != "":
            # create statusInterval element
            statusIntervalElement = doc.createElement("rgma.servicetool.status_interval")
            statusIntervalElement.setAttribute("value", self.statusInterval)
            parametersElement.appendChild(statusIntervalElement)
        
        if self.wsdlUrl != "":
            # create wsdlUrl element
            wsdlUrlElement = doc.createElement("rgma.servicetool.url_wsdl")
            wsdlUrlElement.setAttribute("value", self.wsdlUrl)
            parametersElement.appendChild(wsdlUrlElement)        
        
        if self.semanticsUrl != "":
            # create semanticsUrl element
            semanticsUrlElement = doc.createElement("rgma.servicetool.url_semantics")
            semanticsUrlElement.setAttribute("value", self.semanticsUrl)
            parametersElement.appendChild(semanticsUrlElement)
        
        # create associatedService element
        if self.associatedServices != []:
            associatedServiceElement = doc.createElement("rgma.servicetool.associatedService")
            
            for service in self.associatedServices:                
                # create element
                serviceElement = doc.createElement("value")
                serviceElementValue = doc.createTextNode(service)
                serviceElement.appendChild(serviceElementValue)
                associatedServiceElement.appendChild(serviceElement)
            
            parametersElement.appendChild(associatedServiceElement)
        
        # create param element
        if self.serviceParams != []:
            serviceParamsElement =doc.createElement("rgma.servicetool.param")
            
            for param in self.serviceParams:                
                # create element
                paramElement = doc.createElement("value")
                paramElementValue = doc.createTextNode(param)
                paramElement.appendChild(paramElementValue)
            
            parametersElement.appendChild(serviceParamsElement)

        # create param element
        if self.vos != []:
            voElement = doc.createElement("rgma.servicetool.vo")
            
            for vo in self.vos:                
                # create element
                voElement = doc.createElement("value")
                voElementValue = doc.createTextNode(vo)
                voElement.appendChild(voElementValue)
            
            parametersElement.appendChild(voElement)        
        
        # add instance to configuration
        self.glib.appendInstance(doc.documentElement)
        
        return 0
